﻿using System.Collections;
using UnityEngine;
using UnityEngine.UI;

public class Type8miniManagerV3 : CraneManagerV3
{
	private float armPower;
	private bool leverTilted = false;

	BGMPlayer bp;
	Type8ArmController armController;
    private RopeManagerV3 ropeManager;
	Lever lever;
	Timer timer;
    Timer errorTimer;
    public Text limitTimedisplayed;
	[SerializeField] TextMesh credit3d;

	private float[] CraneLimit = new float[4] {0.355f, 0f, 0.35f, 0f };
	private float[] CraneLimitInGame = new float[4] {0.355f, 0.156f, 0.35f, 0f };

	private Vector2 startPosition;
    private Vector2 releasePosition;
	private float ropeMinDistance = 0.2f;
	private float ropeMaxDistance = 0.48f;
	private float ropeMaxDistanceRelease = 0.28f;

    // 以下config
    [SerializeField] float[] armPowerConfig = new float[3];
    [SerializeField] float[] armPowerConfigSuccess = new float[3];
	[SerializeField] int limitTimeSet = 60;
	[SerializeField] bool downStop = true;
	[SerializeField] int soundType = 0; // 0:HappyCraneAlpha(reserved)

	// TODO: 待機BGMを再生停止するまでの時間

	async void Start()
	{
        Transform temp = transform.Find("CraneUnit").transform;

        craneStatus = -1;
		craneType = 8;
		host.manualCode = 10;

		canvas = transform.Find("Canvas").gameObject;
		creditSystem = transform.Find("CreditSystem").GetComponent<CreditSystemV3>();
		bp = transform.Find("BGM").GetComponent<BGMPlayer>();
        sp = transform.Find("SE").GetComponent<SEPlayer>();
		getPoint = transform.Find("Floor").Find("GetPoint").GetComponent<GetPointV3>();
		lever = transform.Find("Canvas").Find("ControlGroup").Find("Lever").GetComponent<Lever>();
        timer = transform.Find("Timer").GetComponent<Timer>();
        errorTimer = transform.Find("ErrorTimer").GetComponent<Timer>();

        // ロープとアームコントローラに関する処理
        ropeManager = transform.Find("RopeManager").GetComponent<RopeManagerV3>();
        armController = temp.Find("ArmUnit").GetComponent<Type8ArmController>();
		timer.limitTime = limitTimeSet;

        // CraneBoxに関する処理
        craneBox = temp.Find("CraneBox").GetComponent<CraneBoxV3>();

		getPoint.SetManager(this);

        //startPosition = new Vector2(0.156f, 0.178f);
        startPosition = new Vector2(0.156f, 0.34f);
        releasePosition = new Vector2(0, 0.34f);

		StartCoroutine(DelayCoroutine(1000, () => { Setup(0); }));
    }

	public void Setup(int num)
	{
        Transform temp = transform.Find("CraneUnit").transform;

		if (isHibernate)
		{
			creditSystem.SetHibernate();
		}

        creditSystem.Setup();
        creditSystem.SetSEPlayer(sp);
        creditSystem.SetSoundNum(0);

        getSoundNum = 5;

        // TODO: Configクラス作成に備えた一時的な引数
        StartCoroutine(Setup());
	}

	IEnumerator Setup()
	{
        // ロープ巻取り確認
        ropeManager.SetLimits(ropeMaxDistanceRelease, ropeMinDistance);

        if (!ropeManager.CheckPos(2))
        {
            ropeManager.Down(true);
            while (!ropeManager.CheckPos(2))
            {
                yield return null;
            }
        }

        yield return new WaitForSeconds(0.5f);

        if (!ropeManager.CheckPos(1))
		{
			ropeManager.Up(true);
			while (!ropeManager.CheckPos(1))
			{
				yield return null;
			}
		}

        ropeManager.SetLimits(ropeMaxDistance, ropeMinDistance);

        // 左リミット解除(暫定)
        craneBox.SetLimits(CraneLimit[0], CraneLimit[1], CraneLimit[2], CraneLimit[3]);

		// アームユニット位置確認
		if (!craneBox.CheckPos(1))
		{
			craneBox.Left(true);
			craneBox.Forward(true);
			while (!craneBox.CheckPos(1))
			{
				yield return null;
			}
		}

		//armController.Open();

		craneStatus = 0;
	}

	void Update()
	{
		if (useUI && host.playable && !canvas.activeSelf)
		{
			canvas.SetActive(true);
		}
		else if (!host.playable && canvas.activeSelf)
		{
			canvas.SetActive(false);
		}

		if ((Input.GetKeyDown(KeyCode.Keypad0) || Input.GetKeyDown(KeyCode.Alpha0)))
		{
			InsertCoin();
		}
		if (Input.GetKeyDown(KeyCode.R))
		{
			// 再起動
            bp.Stop(0);
            bp.Stop(1);
			//Start();
			Setup(0);
		}

		if (craneStatus == 1)
		{
			if (craneBox.CheckPos(9))
			{
				craneStatus = 2;
			}
		}
		else if (craneStatus == 3 && timer.limitTimeNow <= 0)
		{
			// 操作時間切れ
			craneStatus = 4;
		}
		else if (craneStatus == 4)
		{
			if (ropeManager.CheckPos(2))
			{
				craneStatus = 5;
			}
		}
		else if (craneStatus == 6)
		{
            if (ropeManager.CheckPos(1))
            {
                craneStatus = 7;
            }
        }
        else if (craneStatus == 8)
        {
            if (craneBox.CheckPos(8))
            {
                craneStatus = 9;
            }
        }
        else if (craneStatus == 9)
		{
			if (craneBox.CheckPos(9))
			{
                craneStatus = 10;
            }
		}
		else if (craneStatus == 10)
		{
			if (ropeManager.CheckPos(2))
			{
				craneStatus = 11;
			}
		}
        else if (craneStatus == 12)
        {
            if (ropeManager.CheckPos(1))
            {
                craneStatus = 13;
            }
        }
        else if (craneStatus == 14)
		{
			if (craneBox.CheckPos(9))
			{
				craneStatus = 15;
			}
		}

		if (craneStatus >= 2 && craneStatus <= 4)
		{
			DetectKey(craneStatus);
		}
		if (craneStatus >= 2 && craneStatus <= 3)
		{
			DetectLever();
		}

		if (!creditSystem.segUpdateFlag)
		{
            if (timer.limitTimeNow >= 0)
            {
                limitTimedisplayed.text = timer.limitTimeNow.ToString("D2");
                credit3d.text = timer.limitTimeNow.ToString("D2");
            }
            else
            {
                limitTimedisplayed.text = "00";
                credit3d.text = "00";
            }
        }

        if (errorTimer.limitTimeNow == 0 && craneStatus != 99) // Error
        {
            craneStatus = 99;
        }
    }

	void FixedUpdate()
	{
		if (craneStatus != 0)
		{
			if (craneStatus == 6)
			{
				float power = probability ? armPowerConfigSuccess[1] : armPowerConfig[1];
                if (armPower > power)
                {
                    power -= 0.1f;
                }
            }
			else if (craneStatus == 8)
			{
                float power = probability ? armPowerConfigSuccess[2] : armPowerConfig[2];
				if (armPower > power)
				{
					power -= 0.2f;
				}
				armController.SetMotorPower(power);
            }
			else if (craneStatus == 9)
			{
                float power = probability ? armPowerConfigSuccess[2] : armPowerConfig[2];
                if (armPower > power)
                {
                    power -= 0.2f;
                }
                armController.SetMotorPower(power);
            }
		}
	}

	public override void GetPrize()
	{
        base.GetPrize();
	}

	protected override async void FirstStatusEvent(int status)
	{
		if (status == 0)
		{
			// 待機
			switch (soundType)
			{
				case 0:
					bp.Stop(1);
					bp.Play(0);
					break;

				case 1:
                    bp.Stop(3);
                    bp.Play(2);
					break;
			}
		}
		else if (status == 1)
		{
			// スタート地点に移動
			// 再生されていないならBGMを再生
            switch (soundType)
            {
                case 0:
					if (!bp.audioSources[1].isPlaying)
					{
						bp.Stop(0);
						bp.Play(1);
					}
                    break;

                case 1:
					if (!bp.audioSources[2].isPlaying)
					{
						bp.Stop(3);
						bp.Play(2);
					}
                    break;
            }

			// スタート地点に移動
			craneBox.GoPosition(startPosition);

            // TODO: 移動範囲を制限
            // 正式にCraneBoxV3に左サポートが来るまでの繋ぎ、現状実機で左に行けた試しが無いためという2つの意味
            craneBox.SetLimits(CraneLimitInGame[0], CraneLimitInGame[1], CraneLimitInGame[2], CraneLimitInGame[3]);
        }
		else if (status == 2)
		{
			// 操作待ち
			craneBox.CancelGoPosition();
        }
		else if (status == 3)
		{
			// 操作中
			timer.StartTimer();
			creditSystem.segUpdateFlag = false;

			creditSystem.ResetPay();
			int credit = creditSystem.DecrementCredit();
			creditSystem.NewPlay();

            if (credit < 10) credit3d.text = credit.ToString();
            else credit3d.text = "9.";
        }
		else if (status == 4)
		{
			// 下降中
			// 移動を強制停止
			craneBox.Left(false);
			craneBox.Right(false);
			craneBox.Back(false);
			craneBox.Forward(false);

			// 音声を鳴らす
			switch (soundType)
			{
				case 0:
					sp.Play(1, 1);
					break;
				case 1:
					break;
			}

			timer.CancelTimer();
			creditSystem.segUpdateFlag = true;

			int credit = creditSystem.GetCreditCount();
            if (credit < 100)
            {
                limitTimedisplayed.text = credit.ToString("D2");
                credit3d.text = credit.ToString("D2");
            }
            else
            {
                limitTimedisplayed.text = "99";
                credit3d.text = "99.";
            }

            StartCoroutine(DelayCoroutine(200, () =>
            {
				ropeManager.Down(true);
            }));
        }
		else if (status == 5)
		{
			// 掴む
			ropeManager.Down(false);

			switch (soundType)
			{
				case 0:
					sp.Stop(1);
					sp.Play(2, 1);
					break;
			}

			armController.Close();

			// 確率によるパワー調整
			float power = probability ? armPowerConfigSuccess[0] : armPowerConfig[0];

            if (power < 30)
			{
				power = 30f;
			}
			armController.SetMotorPower(power);

			StartCoroutine(DelayCoroutine(1000, () =>
			{
				craneStatus = 6;
			}));
		}
		else if (status == 6)
		{
			// 上昇中
			ropeManager.Up(true);
            errorTimer.StartTimer();

            switch (soundType)
            {
                case 0:
                    sp.Stop(2);
                    sp.Play(3, 1);
                    break;
            }
        }
		else if (status == 7)
		{
			// 上昇停止
			ropeManager.Up(false);
            errorTimer.CancelTimer();

            switch (soundType)
            {
                case 0:
                    sp.Stop(3);
                    sp.Play(4, 1);
                    break;
            }

			// TODO: パワー調整
			float power = probability ? armPowerConfigSuccess[1] : armPowerConfig[1];
			armController.SetMotorPower(power);

            craneStatus = 8;
		}
		else if (status == 8)
		{
			// 奥まで移動
			craneBox.Back(true);

            // 移動制限解除
            craneBox.SetLimits(CraneLimit[0], CraneLimit[1], CraneLimit[2], CraneLimit[3]);

			// 下降制限設定
			// GoPosition対応までの暫定処理
			ropeManager.SetLimits(ropeMaxDistanceRelease, ropeMinDistance);

            // TODO: 効果音
		}
		else if (status == 9)
		{
			// 離し位置に移動開始
			craneBox.GoPosition(releasePosition);
		}
		else if (status == 10)
		{
			// 下降中
			ropeManager.Down(true);
		}
		else if (status == 11)
		{
			ropeManager.Down(false);

			// 離す
			armController.Open();

            StartCoroutine(DelayCoroutine(1000, () =>
            {
                craneStatus = 12;
            }));
        }
		else if (status == 12)
		{
			// 上昇中
			ropeManager.Up(true);
            errorTimer.StartTimer();
        }
		else if (status == 13)
		{
			// 上昇停止
			ropeManager.Up(false);
            errorTimer.CancelTimer();

            // 下降制限を戻す
            ropeManager.SetLimits(ropeMaxDistance, ropeMinDistance);

            StartCoroutine(DelayCoroutine(1000, () =>
            {
                craneStatus = 14;
            }));
		}
		else if (status == 14)
		{
            // 原点に移動
            // TODO: ファンタジースリムでは残クレジットがあれば4秒待機してプール左奥、
            //       残クレジットがなければ待たずに原点
            if (creditSystem.GetCreditCount() > 0)
            {
				craneBox.GoPosition(startPosition);
            }
            else
            {
				craneBox.GoPosition(new Vector2(0, 0));
            }
		}
		else if (status == 15)
		{
            StartCoroutine(DelayCoroutine(500, () =>
            {
                if (creditSystem.GetCreditCount() > 0)
                {
                    craneStatus = 1;
                }
                else
                {
                    craneStatus = 0;
                }
            }));
        }
		else if (status == 99)
		{
            Debug.Log("Type8 Mini Error! CraneStatus: " + craneStatus);
            
			// ロープ停止
			ropeManager.Up(false);
            ropeManager.Down(false);

			// セグ表示
            limitTimedisplayed.text = "Er";
            creditSystem.Credit.text = "Er";
            credit3d.text = "Er";

			// 音声停止
			bp.Stop(1);
			sp.Stop(3);
        }
    }

	protected override void LastStatusEvent(int status)
	{
		
	}

    protected override void DetectKey(int _status)
	{
		if (host.playable)
		{
			switch (_status)
			{
				case 3:
                    if ((Input.GetKeyDown(KeyCode.Keypad2) || Input.GetKeyDown(KeyCode.Alpha2)))
					{
						ButtonDown(2);
					}
                    break;
				case 4:
                    if ((Input.GetKeyDown(KeyCode.Keypad2) || Input.GetKeyDown(KeyCode.Alpha2)))
					{
						ButtonDown(2);
					}
					break;
			}
		}
	}

	protected void DetectLever()
	{
		if (host.playable)
		{
            if (Input.GetKeyDown(KeyCode.H) || lever.rightFlag)
                craneBox.Right(true);
            else if (Input.GetKeyUp(KeyCode.H) && !lever.rightFlag)
                craneBox.Right(false);

            if (Input.GetKeyDown(KeyCode.F) || lever.leftFlag)
                craneBox.Left(true);
            else if (Input.GetKeyUp(KeyCode.F) && !lever.leftFlag)
                craneBox.Left(false);

            if (Input.GetKeyDown(KeyCode.T) || lever.backFlag)
                craneBox.Back(true);
            else if (Input.GetKeyUp(KeyCode.T) && !lever.backFlag)
                craneBox.Back(false);

            if (Input.GetKeyDown(KeyCode.G) || lever.forwardFlag)
                craneBox.Forward(true);
            else if (Input.GetKeyUp(KeyCode.G) && !lever.forwardFlag)
                craneBox.Forward(false);


            if (Input.GetKey(KeyCode.H) || Input.GetKey(KeyCode.F) || Input.GetKey(KeyCode.T) || Input.GetKey(KeyCode.G)
				|| lever.rightFlag || lever.leftFlag || lever.backFlag || lever.forwardFlag)
			{
                if (craneStatus == 2)
                {
					craneStatus = 3;
                }
			}
        }
	}

	public override void ButtonDown(int _buttonNum)
	{
		switch (_buttonNum)
		{
			case 2:
				if (craneStatus == 3)
				{
					craneStatus = 4;
				}
				else if (craneStatus == 4 && downStop)
				{
					ropeManager.Down(false);
					craneStatus = 5;
				}
				break;
		}
	}

	public override void ButtonUp(int _buttonNum)
	{

	}

	public override void InsertCoin()
	{
		if (!isHibernate && host.playable && craneStatus >= 0)
		{
			int credit = creditSystem.Pay(100);
			credit3d.text = credit < 10 ? credit.ToString() : "9.";
			if (credit > 0 && craneStatus == 0)
			{
				craneStatus = 1;
			}
        }
	}

	public override void InsertCoinAuto()
	{

	}
}
